#include <cstdio>
#include <cstring>
#include <stdexcept>
#include <unistd.h>
#include <pwd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <assert.h>
#include <sqlite3.h>

sqlite3* _sqlite3 = nullptr;

bool directory_exists(const char* fs_path)	
{
	struct stat ss;
	if (stat(fs_path, &ss) != 0)
		return false;
	else
		return (ss.st_mode & S_IFMT) == S_IFDIR;
}

bool file_exists(const char* file_path)	
{
	struct stat ss;
	if (stat(file_path, &ss) != 0)
		return false;
	else
		return (ss.st_mode & S_IFMT) == S_IFREG;
}

std::string get_cesta_db()
{
	std::string db_file = getenv("HOME");
	if (!directory_exists(db_file.c_str()))
		db_file = getpwuid(getuid())->pw_dir;
	if (!directory_exists(db_file.c_str()))
		throw std::runtime_error("Nepodařilo se zjistit domovský adresář.");
	db_file.append("/Dokumenty/test.db");
	return db_file;
}

void vytvorit_databazi()
{
	std::string str_file = get_cesta_db();
	if (file_exists(str_file.c_str()))
	{
		if (remove(str_file.c_str()) !=0)
			throw std::runtime_error("Chyba smazání souboru");
	}
	int ires = sqlite3_open(str_file.c_str(), &_sqlite3);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
	ires = sqlite3_exec(_sqlite3, "CREATE TABLE IF NOT EXISTS polozky (id INTEGER PRIMARY KEY,"
		" cislo INTEGER DEFAULT 0, nazev TEXT)", nullptr, nullptr, nullptr);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
}

void otevrit_databazi()
{
	std::string str_file = get_cesta_db();
	if (!file_exists(str_file.c_str()))
		throw std::runtime_error("Soubor databáze nebyl nalezen");
	int ires = sqlite3_open(str_file.c_str(), &_sqlite3);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
}

void naplnit_nahodne(size_t pocet = 100)
{
	assert(_sqlite3);
	srand(time(nullptr));
	char text[255];
	int cislo;
	int ires;
	for (size_t i = 0; i < pocet; i++)
	{
		cislo = rand() % 100 + 1;
		sprintf(text, "INSERT INTO polozky (cislo, nazev) VALUES (%d, '%d - %X')", cislo, cislo, cislo);
		ires = sqlite3_exec(_sqlite3, text, nullptr, nullptr, nullptr);
		if (ires != SQLITE_OK)
			throw std::runtime_error(sqlite3_errstr(ires));
	}
}

int select_callback(void* p, int count, char** data, char** col_name)
{
	static bool first_line = true;
	if (p && first_line)
	{
		printf("%s\n", (const char*)p);
		first_line = false;
	}
	for (size_t i = 0; i < count; i++)
	{
		printf("%s: %s \t", *(col_name+i), *(data+i));
	}
	printf("\n");
	if (strcmp(*(data+1), "78") == 0)
	{
		printf("Nalezeno číslo 78 na pozici %s\nKončím výpis\n", (char*)(*data));
		return 1;
	}
	return 0;
}

int main(int argc, char** argv)
{
	try
	{
		//vytvorit_databazi();
		//naplnit_nahodne();
		otevrit_databazi();
		char* err_msg = nullptr;
		int ires = sqlite3_exec(_sqlite3, "SELECT * FROM polozky",
			select_callback, (void*)"Ukázka výpisu dat", &err_msg);
		if (err_msg)
		{
			printf("err_msg: %s\n", err_msg);
			sqlite3_free(err_msg);
		}
		printf("sqlite3_errstr: %s\n", sqlite3_errstr(ires));
		if (SQLITE_ABORT == ires)
		{
		}
		else
		{
			if (ires != SQLITE_OK)
				throw std::runtime_error(sqlite3_errstr(ires));
		}
		sqlite3_close(_sqlite3);
		printf("\nok\n");
		return EXIT_SUCCESS;
	}
	catch(const std::exception& e)
	{
		printf("Došlo k chybě: %s\n", e.what());
		if (_sqlite3)
			sqlite3_close(_sqlite3);
		return EXIT_FAILURE;
	}
}

